package bpm.util;

import java.util.ArrayList;
import java.util.List;


import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.tool.hbm2ddl.SchemaExport;

/*
 * @author Kiran Sadasivuni
 */

public class JBPM_v5_1_SchemaGenerator {

	private AnnotationConfiguration cfg;

	public JBPM_v5_1_SchemaGenerator() throws Exception {
		cfg = new AnnotationConfiguration();
		cfg.setProperty("hibernate.hbm2ddl.auto", "create");
	}

	private void generate(Dialect dialect) {
		cfg.setProperty("hibernate.dialect", dialect.getDialectClass());

		SchemaExport export = new SchemaExport(cfg);
		export.setDelimiter(";");
		export.setOutputFile("ddl_" + dialect.name().toLowerCase() + ".sql");
		export.execute(true, false, false, false);
	}
	
	private void addClassesToConfig(List<Class> classes) {
		for (Class clazz : classes) {
			System.out.println("Looking into class " + clazz.toString());
			cfg.addAnnotatedClass(clazz);
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		
		JBPM_v5_1_SchemaGenerator converter = new JBPM_v5_1_SchemaGenerator();
		
		List<Class> classes = converter.getClasses();
		converter.addClassesToConfig(classes);
		converter.generate(Dialect.MYSQL);
		converter.generate(Dialect.ORACLE);
		converter.generate(Dialect.HSQL);
		System.out.println("Program completed");
	}

	private List<Class> getClasses() throws Exception {
		List<Class> classes = new ArrayList<Class>();
		
		classes.add(Class.forName("org.jbpm.persistence.processinstance.ProcessInstanceInfo"));
		classes.add(Class.forName("org.drools.persistence.info.SessionInfo"));
		classes.add(Class.forName("org.drools.persistence.info.WorkItemInfo"));
		classes.add(Class.forName("org.jbpm.process.audit.ProcessInstanceLog"));
		classes.add(Class.forName("org.jbpm.process.audit.NodeInstanceLog"));
		classes.add(Class.forName("org.jbpm.process.audit.VariableInstanceLog"));
		
		classes.add(Class.forName("org.jbpm.task.Task"));
		classes.add(Class.forName("org.jbpm.task.Comment"));
		classes.add(Class.forName("org.jbpm.task.Attachment"));
		classes.add(Class.forName("org.jbpm.task.I18NText"));
		classes.add(Class.forName("org.jbpm.task.SubTasksStrategy"));
		classes.add(Class.forName("org.jbpm.task.Deadline"));
		classes.add(Class.forName("org.jbpm.task.Escalation"));
		classes.add(Class.forName("org.jbpm.task.Reassignment"));
		classes.add(Class.forName("org.jbpm.task.Notification"));
		classes.add(Class.forName("org.jbpm.task.BooleanExpression"));
		classes.add(Class.forName("org.jbpm.task.User"));
		classes.add(Class.forName("org.jbpm.task.PeopleAssignments"));
		
		classes.add(Class.forName("org.jbpm.task.Delegation"));
		classes.add(Class.forName("org.jbpm.task.Group"));
		classes.add(Class.forName("org.jbpm.task.EmailNotification"));
		classes.add(Class.forName("org.jbpm.task.EmailNotificationHeader"));
		classes.add(Class.forName("org.jbpm.task.Content"));
		classes.add(Class.forName("org.jbpm.task.Status"));
		classes.add(Class.forName("org.jbpm.task.TaskData"));
		classes.add(Class.forName("org.jbpm.task.OnParentAbortAllSubTasksEndStrategy"));
		classes.add(Class.forName("org.jbpm.task.OnAllSubTasksEndParentEndStrategy"));
		return classes;
	}

	/**
	 * Holds the classnames of hibernate dialects for easy reference.
	 */
	private static enum Dialect {
		ORACLE("org.hibernate.dialect.OracleDialect"), 
		MYSQL("org.hibernate.dialect.MySQLDialect"), 
		HSQL("org.hibernate.dialect.HSQLDialect");

		private String dialectClass;

		private Dialect(String dialectClass) {
			this.dialectClass = dialectClass;
		}

		public String getDialectClass() {
			return dialectClass;
		}
	}	

}
